#ifndef LB_D3Q19_VDW_H
#define LB_D3Q19_VDW_H

#include <lb-VdW.h>
#include <lb-macros.h>
#include <lb-partition-info.h>

LB_BEGIN_DECLS

typedef struct {
	double min_density;
	double max_density;
	double max_velocity;
	double kin_energy;
	double momentum_x;
	double momentum_y;
	double momentum_z;
} LBD3Q19VdWStats;

typedef struct _LBD3Q19VdW LBD3Q19VdW;

LBD3Q19VdW* lb_d3q19_VdW_new(int nx, int ny, int nz, int pz);
void lb_d3q19_VdW_destroy(LBD3Q19VdW*);

void lb_d3q19_VdW_partition_info(const LBD3Q19VdW*, LBPartitionInfo*);

void lb_d3q19_VdW_set_parameters(LBD3Q19VdW*, const LBVdWParameters*);
void lb_d3q19_VdW_get_parameters(const LBD3Q19VdW*, LBVdWParameters*);

void lb_d3q19_VdW_set_walls_speed(LBD3Q19VdW*, const double v4[2],
						   const double v5[2]);
void lb_d3q19_VdW_get_walls_speed(const LBD3Q19VdW*, double*, double*);

void lb_d3q19_VdW_set_averages(LBD3Q19VdW*, int x, int y, int z,
    double density, double velocity_x, double velocity_y, double velocity_z);

void lb_d3q19_VdW_get_averages(const LBD3Q19VdW*, int x, int y, int z,
    double* density, double* velocity_x, double* velocity_y, double* velocity_z);

void lb_d3q19_VdW_setup_equilibrium_pdfs(LBD3Q19VdW*);
void lb_d3q19_VdW_advance(LBD3Q19VdW*);
void lb_d3q19_VdW_dump(const LBD3Q19VdW*, const char* filename);

void lb_d3q19_VdW_stats(const LBD3Q19VdW*, LBD3Q19VdWStats*);

// remove this
double lb_d3q19_VdW_mass(const LBD3Q19VdW*);

LB_END_DECLS

#endif /* LB_D3Q19_VDW_H */
