#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include "lb-messages.h"
#include "lblua-class.h"

void
lblua_register_class(lua_State*      ls,
		       const char*     modulename,
		       const char*     classname,
		       lua_CFunction   ctor,
		       lua_CFunction   dtor,
		       const luaL_reg* methods)
{
	int moduletable;
	int methodtable;
	int metatable;

	lb_assert(ls != NULL);
	lb_assert(classname != NULL);
	lb_assert(ctor != NULL);
	lb_assert(dtor != NULL);
	lb_assert(methods != NULL);

	if (modulename != NULL) {

		/*
		 *  check if 'modulename' already exists
		 */

		lua_pushstring(ls, modulename);
		lua_gettable(ls, LUA_GLOBALSINDEX);

		/*
		 *  create 'modulename' table if necessary
		 */

		if (lua_isnil(ls, -1)) {
			lua_pop(ls, 1);
			lua_newtable(ls);
			lua_pushstring(ls, modulename);
			lua_pushvalue(ls, -2);
			lua_settable(ls, LUA_GLOBALSINDEX);
		}
	}

	/*
	 *  if 'modulename' == NULL we expect table on top of the stack
	 */

	lb_assert(lua_istable(ls, -1));
	moduletable = lua_gettop(ls);

	/*
	 *  method table
	 */

	lua_newtable(ls);
	methodtable = lua_gettop(ls);

	/*
	 *  metatable
	 */

	luaL_newmetatable(ls, classname);
	metatable = lua_gettop(ls);

	lua_pushliteral(ls, "__metatable");
	lua_pushvalue(ls, methodtable);
	lua_settable(ls, metatable);

	lua_pushliteral(ls, "__index");
	lua_pushvalue(ls, methodtable);
	lua_settable(ls, metatable);

	lua_pushliteral(ls, "__gc");
	lua_pushcfunction(ls, dtor);
	lua_settable(ls, metatable);

	/* drop metatable */
	lua_pop(ls, 1);

	/* register methods */
	luaL_openlib(ls, NULL, methods, 0);

	/* drop methodtable */
	lua_pop(ls, 1);

	lua_pushstring(ls, classname);
	lua_pushcfunction(ls, ctor);
	lua_settable(ls, moduletable);

	/* drop moduletable if necessary */
	if (modulename != NULL)
		lua_pop(ls, 1);
}

void*
lblua_checkudata(lua_State* ls, int narg, const char* classname)
{
	void* ptr;

	lb_assert(ls != NULL);
	lb_assert(classname != NULL);

	luaL_checktype(ls, narg, LUA_TUSERDATA);

	ptr = luaL_checkudata(ls, narg, classname);
	if (ptr == NULL)
		luaL_typerror(ls, narg, classname);

	return ptr;
}
