#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <lua.h>
#include <lauxlib.h>
#include "lb-messages.h"

#include "lb-d3q19-VdW.h"
#include "lua-lb-d3q19-VdW.h"

#include "lblua-class.h"
#include "lblua-utils.h"
#include "lua-lb-common.h"

#ifndef LUA_LB_D3Q19_VDW_CLASSNAME
#define LUA_LB_D3Q19_VDW_CLASSNAME "d3q19_VdW"
#endif /* LUA_LB_D3Q19_VDW_CLASSNAME */

static int
lua_lb_d3q19_VdW_new(lua_State* ls)
{
	int sx, sy, sz, pz;
	LBD3Q19VdW* VdW;

	lb_assert(ls != NULL);

	lblua_checkargcount(ls, 4, LUA_LB_D3Q19_VDW_CLASSNAME
				     "(int,int,int,bool)");

	sx = luaL_checkint(ls, 1);
	sy = luaL_checkint(ls, 2);
	sz = luaL_checkint(ls, 3);

	if (sx <= 3 || sy <= 3 || sz <= 3)
		luaL_error(ls, "first three arguments of "
			   LUA_LB_D3Q19_VDW_CLASSNAME"(int,int,int,bool) "
			   "should be greater than 3");

	if (!lua_isboolean(ls, 4))
		luaL_error(ls, "last argument of "
			   LUA_LB_D3Q19_VDW_CLASSNAME"(int,int,int,bool)"
			   " should be of boolean type");

	pz = lua_toboolean(ls, 4);

	VdW = lb_d3q19_VdW_new(sx, sy, sz, pz);

	lua_boxpointer(ls, VdW);
	luaL_getmetatable(ls, LUA_LB_D3Q19_VDW_CLASSNAME);
	lua_setmetatable(ls, -2);

	return 1;
}

static int
lua_lb_d3q19_VdW_destroy(lua_State* ls)
{
	LBD3Q19VdW* VdW;

	lb_assert(ls != NULL);
	VdW = lblua_checkinstance(LBD3Q19VdW, ls, 1,
				    LUA_LB_D3Q19_VDW_CLASSNAME);
	lb_d3q19_VdW_destroy(VdW);

	return 0;
}

static int
lua_lb_d3q19_VdW_typename(lua_State* ls)
{
	lb_assert(ls != NULL);

	lblua_checkargcount(ls, 1, LUA_LB_D3Q19_VDW_CLASSNAME
				     ":typename()");
	(void) lblua_checkinstance(LBD3Q19VdW, ls, 1,
				     LUA_LB_D3Q19_VDW_CLASSNAME);
	lua_pushstring(ls, LUA_LB_D3Q19_VDW_CLASSNAME);

	return 1;
}

static int
lua_lb_d3q19_VdW_partition_info(lua_State* ls)
{
	LBD3Q19VdW* VdW;

	lb_assert(ls != NULL);

	lblua_checkargcount(ls, 1, LUA_LB_D3Q19_VDW_CLASSNAME
				     ":partition_info()");
	VdW = lblua_checkinstance(LBD3Q19VdW, ls, 1,
				    LUA_LB_D3Q19_VDW_CLASSNAME);

	lua_push_lb_partition_info(ls, (LBPartitionInfo *) VdW);

	return 1;
}

static void
lua_lb_d3q19_VdW_check_coords(lua_State* ls,
	const LBD3Q19VdW* VdW, int x, int y, int z,
	const char* method)
{
	const LBPartitionInfo* pinfo;

	lb_assert(ls && VdW);

	/* bad hack */
	pinfo = (LBPartitionInfo *) VdW;

	if (x < 0 || x >= pinfo->size[0] ||
	    y < 0 || y >= pinfo->size[1] ||
	    z < 0 || z >= pinfo->size[2])
		lblua_range_error(ls, method);
}

static int
lua_lb_d3q19_VdW_get_averages(lua_State* ls)
{
	LBD3Q19VdW* VdW;
	int x, y, z;

	lb_assert(ls != NULL);

	lblua_checkargcount(ls, 4, LUA_LB_D3Q19_VDW_CLASSNAME
				     ":get_averages(int,int,int)");

	VdW = lblua_checkinstance(LBD3Q19VdW, ls, 1,
				    LUA_LB_D3Q19_VDW_CLASSNAME);

	x = luaL_checkint(ls, 2);
	y = luaL_checkint(ls, 3);
	z = luaL_checkint(ls, 4);

	lua_lb_d3q19_VdW_check_coords(ls, VdW, x, y, z,
		LUA_LB_D3Q19_VDW_CLASSNAME":get_averages(int,int,int)");

	{
		double rho, ux, uy, uz;

		lb_d3q19_VdW_get_averages(VdW, x, y, z,
					    &rho, &ux, &uy, &uz);

		lua_pushnumber(ls, (lua_Number) rho);
		lua_pushnumber(ls, (lua_Number) ux);
		lua_pushnumber(ls, (lua_Number) uy);
		lua_pushnumber(ls, (lua_Number) uz);
	}

	return 4;
}

static int
lua_lb_d3q19_VdW_set_averages(lua_State* ls)
{
	LBD3Q19VdW* VdW;
	int x, y, z;

	lb_assert(ls != NULL);

	lblua_checkargcount(ls, 8, LUA_LB_D3Q19_VDW_CLASSNAME
		 ":set_averages(int,int,int,double,double,double,double)");

	VdW = lblua_checkinstance(LBD3Q19VdW, ls, 1,
				    LUA_LB_D3Q19_VDW_CLASSNAME);

	x = luaL_checkint(ls, 2);
	y = luaL_checkint(ls, 3);
	z = luaL_checkint(ls, 4);

	lua_lb_d3q19_VdW_check_coords(ls, VdW, x, y, z,
		LUA_LB_D3Q19_VDW_CLASSNAME
		":set_averages(int,int,int,double,double,double,double)");

	{
		double rho;

		rho = (double) luaL_checknumber(ls, 5);

		if (rho <= 0.0)
			luaL_error(ls, LUA_LB_D3Q19_VDW_CLASSNAME
			 ":set_averages(int,int,int,double,double,double,double): "
			 " 'rho' (fourth argument) should be positive");


		lb_d3q19_VdW_set_averages(VdW, x, y, z, rho,
					   luaL_checknumber(ls, 6),
					   luaL_checknumber(ls, 7),
					   luaL_checknumber(ls, 8));
	}

	return 0;
}

static int
lua_lb_d3q19_VdW_set_equilibrium(lua_State* ls)
{
	LBD3Q19VdW* VdW;

	lb_assert(ls != NULL);

	lblua_checkargcount(ls, 1, LUA_LB_D3Q19_VDW_CLASSNAME
				     ":set_equilibrium()");
	VdW = lblua_checkinstance(LBD3Q19VdW, ls, 1,
				    LUA_LB_D3Q19_VDW_CLASSNAME);
	lb_d3q19_VdW_setup_equilibrium_pdfs(VdW);

	return 0;
}

static const char* lb_VdW_parameters[5] = {
	"T", "a", "b", "K", "tau"
};

static int
lua_lb_d3q19_VdW_set_parameters(lua_State* ls)
{
	LBD3Q19VdW* VdW;

	lb_assert(ls != NULL);

	lblua_checkargcount(ls, 2, LUA_LB_D3Q19_VDW_CLASSNAME
				     ":set_parameters(table)");
	VdW = lblua_checkinstance(LBD3Q19VdW, ls, 1,
				    LUA_LB_D3Q19_VDW_CLASSNAME);

	{
		LBVdWParameters params;

		lblua_check_simple_table(ls, 2, 5, lb_VdW_parameters,
					   (double*) &params);
		lb_d3q19_VdW_set_parameters(VdW, &params);
	}

	return 0;
}

static int
lua_lb_d3q19_VdW_get_parameters(lua_State* ls)
{
	LBD3Q19VdW* VdW;

	lb_assert(ls != NULL);

	lblua_checkargcount(ls, 1, LUA_LB_D3Q19_VDW_CLASSNAME
				     ":get_parameters()");
	VdW = lblua_checkinstance(LBD3Q19VdW, ls, 1,
				    LUA_LB_D3Q19_VDW_CLASSNAME);

	{
		LBVdWParameters params;

		lb_d3q19_VdW_get_parameters(VdW, &params);
		lblua_push_simple_table(ls, 5, lb_VdW_parameters,
					  (double*) &params);
	}

	return 1;
}

static int
lua_lb_d3q19_VdW_set_walls_speed(lua_State* ls)
{
	LBD3Q19VdW* VdW;

	lb_assert(ls != NULL);

	lblua_checkargcount(ls, 3, LUA_LB_D3Q19_VDW_CLASSNAME
				     ":set_walls_speed(vector,vector)");
	VdW = lblua_checkinstance(LBD3Q19VdW, ls, 1,
				    LUA_LB_D3Q19_VDW_CLASSNAME);
	{
		double v4[2], v5[2];

		if (lblua_check_array_double(ls, 2, 2, v4) != 2)
			luaL_error(ls, LUA_LB_D3Q19_VDW_CLASSNAME
			 ":set_walls_speed(vector,vector): "
			 " unexpected length of the first argument");

		if (lblua_check_array_double(ls, 3, 2, v5) != 2)
			luaL_error(ls, LUA_LB_D3Q19_VDW_CLASSNAME
			 ":set_walls_speed(vector,vector): "
			 " unexpected length of the second argument");


		lb_d3q19_VdW_set_walls_speed(VdW, v4, v5);
	}

	return 0;
}

static int
lua_lb_d3q19_VdW_get_walls_speed(lua_State* ls)
{
	LBD3Q19VdW* VdW;
	double v4[2], v5[2];

	lb_assert(ls != NULL);

	lblua_checkargcount(ls, 1, LUA_LB_D3Q19_VDW_CLASSNAME
				     ":get_walls_speed()");
	VdW = lblua_checkinstance(LBD3Q19VdW, ls, 1,
				    LUA_LB_D3Q19_VDW_CLASSNAME);

	lb_d3q19_VdW_get_walls_speed(VdW, v4, v5);

	lblua_push_array_double(ls, 2, v4);
	lblua_push_array_double(ls, 2, v5);

	return 2;
}

static int
lua_lb_d3q19_VdW_advance(lua_State* ls)
{
	LBD3Q19VdW* VdW;

	lb_assert(ls != NULL);

	lblua_checkargcount(ls, 1, LUA_LB_D3Q19_VDW_CLASSNAME
				     ":advance()");

	VdW = lblua_checkinstance(LBD3Q19VdW, ls, 1,
				    LUA_LB_D3Q19_VDW_CLASSNAME);

	lb_d3q19_VdW_advance(VdW);

	return 0;
}

static int
lua_lb_d3q19_VdW_stats(lua_State* ls)
{
	LBD3Q19VdW* VdW;
	static const char* entries[7] = {
		"min_density", "max_density",
		"max_velocity", "kin_energy",
		"momentum_x", "momentum_y", "momentum_z"
	};

	lb_assert(ls != NULL);

	lblua_checkargcount(ls, 1, LUA_LB_D3Q19_VDW_CLASSNAME
				     ":stats()");
	VdW = lblua_checkinstance(LBD3Q19VdW, ls, 1,
				    LUA_LB_D3Q19_VDW_CLASSNAME);

	{
		LBD3Q19VdWStats stats;

		lb_d3q19_VdW_stats(VdW, &stats);

		lb_assert(sizeof(LBD3Q19VdWStats) == 7*sizeof(double));
		lblua_push_simple_table(ls, 7, entries, (double*) &stats);
	}

	return 1;
}

static int
lua_lb_d3q19_VdW_mass(lua_State* ls)
{
	LBD3Q19VdW* toy;

	lb_assert(ls != NULL);

	lblua_checkargcount(ls, 1, LUA_LB_D3Q19_VDW_CLASSNAME":mass()");
	toy = lblua_checkinstance(LBD3Q19VdW, ls, 1, LUA_LB_D3Q19_VDW_CLASSNAME);
    	lua_pushnumber(ls, lb_d3q19_VdW_mass(toy));

	return 1;
}

static int
lua_lb_d3q19_VdW_dump(lua_State* ls)
{
	LBD3Q19VdW* VdW;

	lb_assert(ls != NULL);

	lblua_checkargcount(ls, 2, LUA_LB_D3Q19_VDW_CLASSNAME
				     ":dump(string)");
	VdW = lblua_checkinstance(LBD3Q19VdW, ls, 1,
				    LUA_LB_D3Q19_VDW_CLASSNAME);
	lb_d3q19_VdW_dump(VdW, luaL_checkstring(ls, 2));

	return 0;
}

void
lua_lb_d3q19_VdW_register(lua_State* ls, const char* modulename)
{
	const luaL_reg methods[] = {
		{"typename",        lua_lb_d3q19_VdW_typename},
		{"partition_info",  lua_lb_d3q19_VdW_partition_info},
		{"get_averages",    lua_lb_d3q19_VdW_get_averages},
		{"set_averages",    lua_lb_d3q19_VdW_set_averages},
		{"set_equilibrium", lua_lb_d3q19_VdW_set_equilibrium},
		{"set_parameters",  lua_lb_d3q19_VdW_set_parameters},
		{"get_parameters",  lua_lb_d3q19_VdW_get_parameters},
		{"set_walls_speed", lua_lb_d3q19_VdW_set_walls_speed},
		{"get_walls_speed", lua_lb_d3q19_VdW_get_walls_speed},
		{"advance",         lua_lb_d3q19_VdW_advance},
		{"stats",           lua_lb_d3q19_VdW_stats},
		{"dump",            lua_lb_d3q19_VdW_dump},
		{"mass",            lua_lb_d3q19_VdW_mass},
		{NULL, NULL}
	};

	lb_assert(ls != NULL);

	lblua_register_class(ls, modulename,
			       LUA_LB_D3Q19_VDW_CLASSNAME,
			       lua_lb_d3q19_VdW_new,
			       lua_lb_d3q19_VdW_destroy,
			       methods);
}
