#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <lua.h>
#include <lauxlib.h>

#ifdef LB_ENABLE_MPI
#include <mpi.h>
#include "lb-mpi-utils.h"
#endif /* LB_ENABLE_MPI */

#include "lb-wtime.h"
#include "lb-memory.h"
#include "lb-messages.h"

#include "lblua-utils.h"
#include "lua-lb-misc.h"

static int
lua_lb_version(lua_State* ls)
{
	lb_assert(ls != NULL);

	lblua_checkargcount(ls, 0, "version()");
	lua_pushstring(ls, PACKAGE_STRING" (c) v0va bab1n 2004");

	return 1;
}

static int
lua_lb_enable_info(lua_State* ls)
{
	lb_assert(ls != NULL);

	lblua_checkargcount(ls, 0, "enable_info()");
	lb_info_enable();

	return 0;
}

static int
lua_lb_disable_info(lua_State* ls)
{
	lb_assert(ls != NULL);

	lblua_checkargcount(ls, 0, "disable_info()");
	lb_info_disable();

	return 0;
}

static int
lua_lb_wtime(lua_State* ls)
{
	lb_assert(ls != NULL);

	lblua_checkargcount(ls, 0, "wtime()");
	lua_pushnumber(ls, lb_wtime());

	return 1;
}

static int
lua_lb_wtime_string(lua_State* ls)
{
	lb_assert(ls != NULL);

	lblua_checkargcount(ls, 1, "wtime_string(number)");
	lua_pushstring(ls, lb_wtime_string(luaL_checknumber(ls, 1)));

	return 1;
}

static int
lua_lb_is_parallel(lua_State* ls)
{
	lb_assert(ls != NULL);

	lblua_checkargcount(ls, 0, "is_parallel()");
#ifdef LB_ENABLE_MPI
	lua_pushboolean(ls, 1);
#else /* !LB_ENABLE_MPI */
	lua_pushboolean(ls, 0);
#endif /* LB_ENABLE_MPI */

	return 1;
}

static int
lua_lb_mpi_barrier(lua_State* ls)
{
	lb_assert(ls != NULL);

	lblua_checkargcount(ls, 0, "mpi_barrier()");

#ifdef LB_ENABLE_MPI
	LB_CALL_MPI(MPI_Barrier(MPI_COMM_WORLD))
#endif /* LB_ENABLE_MPI */

	return 0;
}

void
lua_lb_misc_register(lua_State* ls, const char* modulename)
{
	const luaL_reg lb_misc[] = {
		{"wtime",         lua_lb_wtime},
		{"wtime_string",  lua_lb_wtime_string},
		{"version",	  lua_lb_version},
		{"enable_info",	  lua_lb_enable_info},
		{"disable_info",  lua_lb_disable_info},
		{"is_parallel",   lua_lb_is_parallel},
		{"mpi_barrier",   lua_lb_mpi_barrier},
		{NULL, NULL}
	};

	lb_assert(ls != NULL);

	luaL_openlib(ls, modulename, lb_misc, 0);
}
